@@ -1,10 +1,22 @@
-__version__ = "0.7.17"
+try:
+    from importlib.metadata import version
+    from importlib.metadata import PackageNotFoundError
+except ModuleNotFoundError:
+    # Try backported to PY<38 `importlib_metadata`.
+    from importlib_metadata import version
+    from importlib_metadata import PackageNotFoundError
 
 
 import rdkit
 import packaging.version
 
 
+try:
+    __version__ = version("datamol")
+except PackageNotFoundError:
+    # package is not installed
+    __version__ = "dev"
+
 CURRENT_RDKIT_VERSION = rdkit.__version__
 CURRENT_RDKIT_VERSION_OBJ = packaging.version.parse(CURRENT_RDKIT_VERSION)
 
