@@ -31,6 +31,8 @@ from .descriptors import refractivity
 from .descriptors import n_rigid_bonds
 from .descriptors import n_stereo_centers
 from .descriptors import n_charged_atoms
+from .descriptors import n_stereo_centers_unspecified
+from .descriptors import n_spiro_atoms
 
 from .compute import any_rdkit_descriptor
 from .compute import compute_many_descriptors
