@@ -13,23 +13,14 @@ import hashlib
 
 from loguru import logger
 
+import numpy as np
+import numpy.typing as npt
 from rdkit import Chem
 
-from rdkit.Chem import CanonicalRankAtoms  # type: ignore
-from rdkit.Chem import MolFromSmiles  # type: ignore
-from rdkit.Chem import AddHs  # type: ignore
-from rdkit.Chem import RemoveHs  # type: ignore
-from rdkit.Chem import Kekulize  # type: ignore
-from rdkit.Chem import RenumberAtoms  # type: ignore
-from rdkit.Chem import RWMol  # type: ignore
-from rdkit.Chem import AssignStereochemistry  # type: ignore
-from rdkit.Chem.AllChem import ReplaceSubstructs  # type: ignore
-from rdkit.Chem.AllChem import DeleteSubstructs  # type: ignore
-from rdkit.Chem import GetMolFrags  # type: ignore
-from rdkit.Chem import PathToSubmol  # type: ignore
-
 from rdkit.Chem import rdmolops
 from rdkit.Chem import rdchem
+from rdkit.Chem import rdmolfiles
+from rdkit.Chem import rdGeometry
 
 from rdkit.Chem.Scaffolds import MurckoScaffold
 
@@ -43,6 +34,7 @@ from .convert import to_inchikey
 from .convert import to_smiles
 from .convert import from_smarts
 from .log import without_rdkit_log
+from ._version import is_lower_than_current_rdkit_version
 
 
 PERIODIC_TABLE = Chem.rdchem.GetPeriodicTable()
@@ -70,12 +62,16 @@ def to_mol(
     ordered: bool = False,
     kekulize: bool = False,
     sanitize: bool = True,
+    allow_cxsmiles: bool = True,
+    parse_name: bool = True,
+    remove_hs: bool = True,
+    strict_cxsmiles: bool = True,
 ) -> Optional[Mol]:
     """Convert an input molecule (smiles representation) into a `Mol`.
 
     Args:
         mol: A SMILES or a molecule.
-        add_hs: Whether hydrogens should be added the molecule.
+        add_hs: Whether hydrogens should be added the molecule after the SMILES has been parsed.
         explicit_only: Whether to only add explicit hydrogen or both
             (implicit and explicit). when `add_hs` is set to True.
         ordered: Whether the atom should be ordered. This option is
@@ -83,6 +79,10 @@ def to_mol(
             a single atom order for the same molecule, regardless of its original SMILES representation.
         kekulize: Whether to perform kekulization of the input molecules.
         sanitize: Whether to apply rdkit sanitization when input is a SMILES.
+        allow_cxsmiles: Recognize and parse CXSMILES.
+        parse_name: Parse (and set) the molecule name as well.
+        remove_hs: Wether to remove the hydrogens from the input SMILES.
+        strict_cxsmiles: Throw an exception if the CXSMILES parsing fails.
 
     Returns:
         mol: the molecule if some conversion have been made. If the conversion fails
@@ -93,7 +93,14 @@ def to_mol(
         raise ValueError(f"Input should be a Mol or a string instead of '{type(mol)}'")
 
     if isinstance(mol, str):
-        _mol = MolFromSmiles(mol, sanitize=sanitize)
+        smiles_params = rdmolfiles.SmilesParserParams()
+        smiles_params.sanitize = sanitize
+        smiles_params.allowCXSMILES = allow_cxsmiles
+        smiles_params.parseName = parse_name
+        smiles_params.removeHs = remove_hs
+        smiles_params.strictCXSMILES = strict_cxsmiles
+
+        _mol = rdmolfiles.MolFromSmiles(mol, params=smiles_params)
 
         if not sanitize and _mol is not None:
             _mol.UpdatePropertyCache(False)
@@ -102,14 +109,15 @@ def to_mol(
 
     # Add hydrogens
     if _mol is not None and add_hs:
-        _mol = AddHs(_mol, explicitOnly=explicit_only, addCoords=True)
+        _mol = rdmolops.AddHs(_mol, explicitOnly=explicit_only, addCoords=True)
 
     # Reorder atoms
     if _mol is not None and ordered:
         _mol = reorder_atoms(_mol)
 
     if _mol is not None and kekulize:
-        Kekulize(_mol, clearAromaticFlags=False)
+        rdmolops.Kekulize(_mol, clearAromaticFlags=False)
+
     return _mol
 
 
@@ -155,6 +163,52 @@ def unique_id(mol: Mol) -> Optional[str]:
     return hashlib.md5(ik.encode("utf-8")).hexdigest()
 
 
+def hash_mol(mol: Mol, hash_scheme: str = "all") -> str:
+    """Generate a unique hash code for a molecule based on chemistry. If two molecules are
+    chemically “the same”, they should have the same hash.
+
+    Using molhash adds value beyond using SMILES because it:
+
+    - Ignores SMILES features that are not chemically meaningful (e.g. atom map numbers).
+    - Canonicalizes enhanced stereochemistry groups. For example `C[C@H](O)CC |&1:1|` and `C[C@@H](O)CC |&1:1|`
+    have the same molhash.
+    - Canonicalizes S group data (for example, polymer data).
+
+    Args:
+        mol: A molecule.
+        hash_scheme: There are three hash schemes:
+            - "all": most strict hash scheme utilizing all layers.
+            - "no_stereo": excludes stereo sensitive layers.
+            - "no_tautomers": excludes tautomer sensitive layers.
+
+    Returns:
+        The hash as a 40 chars string.
+    """
+
+    if is_lower_than_current_rdkit_version("2022.09"):
+        raise NotImplementedError("`datamol.hash_mol() is only available with RDKit>=2022.09.")
+
+    from rdkit.Chem import RegistrationHash
+
+    # Compute the mol layers
+    all_layers = RegistrationHash.GetMolLayers(mol)
+
+    # Select the hash scheme
+    if hash_scheme == "all":
+        hash_scheme_enum = RegistrationHash.HashScheme.ALL_LAYERS
+    elif hash_scheme == "no_stereo":
+        hash_scheme_enum = RegistrationHash.HashScheme.STEREO_INSENSITIVE_LAYERS
+    elif hash_scheme == "no_tautomers":
+        hash_scheme_enum = RegistrationHash.HashScheme.TAUTOMER_INSENSITIVE_LAYERS
+    else:
+        raise ValueError(
+            f"`hash_scheme` is invalid. Please choose from: 'all', 'no_stereo' or 'no_tautomers'."
+        )
+
+    # Generate the hash
+    return RegistrationHash.GetMolHash(all_layers=all_layers, hash_scheme=hash_scheme_enum)
+
+
 def reorder_atoms(
     mol: Mol,
     break_ties: bool = True,
@@ -176,14 +230,14 @@ def reorder_atoms(
     if mol.GetNumAtoms() == 0:
         return mol
 
-    new_order = CanonicalRankAtoms(
+    new_order = rdmolfiles.CanonicalRankAtoms(
         mol,
         breakTies=break_ties,
         includeChirality=include_chirality,
         includeIsotopes=include_isotopes,
     )
     new_order = sorted([(y, x) for x, y in enumerate(new_order)])
-    return RenumberAtoms(mol, [y for (x, y) in new_order])
+    return rdmolops.RenumberAtoms(mol, [y for (x, y) in new_order])
 
 
 def randomize_atoms(mol: Mol) -> Optional[Mol]:
@@ -200,7 +254,7 @@ def randomize_atoms(mol: Mol) -> Optional[Mol]:
 
     atom_indices = list(range(mol.GetNumAtoms()))
     random.shuffle(atom_indices)
-    return RenumberAtoms(mol, atom_indices)
+    return rdmolops.RenumberAtoms(mol, atom_indices)
 
 
 def to_neutral(mol: Optional[Mol]) -> Optional[Mol]:
@@ -269,7 +323,6 @@ def sanitize_mol(
         mol = _sanifix4.sanifix(mol)
 
     if mol is not None:
-
         # Detect multiple conformers
         if verbose and mol.GetNumConformers() > 1:
             logger.warning(
@@ -433,7 +486,7 @@ def standardize_mol(
         mol = uncharger.uncharge(mol)
 
     if stereo:
-        AssignStereochemistry(mol, force=False, cleanIt=True)
+        rdmolops.AssignStereochemistry(mol, force=False, cleanIt=True)
 
     return mol
 
@@ -453,7 +506,6 @@ def fix_valence_charge(mol: Mol, inplace: bool = False) -> Optional[Mol]:
 
     # Don't fix something that is not broken
     if len(vm.validate(mol)) > 0:
-
         if not inplace:
             mol = copy.copy(mol)
 
@@ -546,7 +598,7 @@ def fix_valence(mol: Mol, inplace: bool = False, allow_ring_break: bool = False)
                 # atom.SetNumRadicalElectrons(0)
             atom.UpdatePropertyCache(False)
 
-        em = RWMol(m)
+        em = rdchem.RWMol(m)
         bonds = em.GetBonds()
         bonds = [
             bond
@@ -586,7 +638,7 @@ def adjust_singleton(mol: Mol) -> Optional[Mol]:
         mol: a molecule.
     """
     to_rem = []
-    em = RWMol(mol)
+    em = rdchem.RWMol(mol)
     for atom in mol.GetAtoms():
         if atom.GetExplicitValence() == 0:
             to_rem.append(atom.GetIdx())
@@ -603,10 +655,10 @@ def remove_dummies(mol: Mol, dummy: str = "*") -> Optional[Mol]:
     out = mol
 
     try:
-        out = ReplaceSubstructs(mol, du, to_mol("[H]"), True)[0]
+        out = rdmolops.ReplaceSubstructs(mol, du, to_mol("[H]"), True)[0]
         out = remove_hs(out)
     except Exception:
-        out = DeleteSubstructs(mol, du)
+        out = rdmolops.DeleteSubstructs(mol, du)
     return out
 
 
@@ -669,14 +721,14 @@ def replace_dummies_atoms(
     """
     du = to_mol(dummy)
     replacement = to_mol(atom)
-    out = ReplaceSubstructs(mol, du, replacement, replaceAll=replace_all)[0]
+    out = rdmolops.ReplaceSubstructs(mol, du, replacement, replaceAll=replace_all)[0]
     return out
 
 
 def keep_largest_fragment(mol: Mol) -> Optional[Mol]:
     """Only keep largest fragment of each molecule."""
     return max(
-        GetMolFrags(mol, asMols=True),
+        rdmolops.GetMolFrags(mol, asMols=True),
         default=mol,
         key=lambda m: m.GetNumAtoms(),
     )
@@ -704,7 +756,7 @@ def set_dative_bonds(mol: Mol, from_atoms: Tuple[int, int] = (7, 8)) -> Optional
     Returns:
         The modified molecule.
     """
-    rwmol = RWMol(mol)
+    rwmol = rdchem.RWMol(mol)
     rwmol.UpdatePropertyCache(strict=False)
 
     metals = [at for at in rwmol.GetAtoms() if is_transition_metal(at)]
@@ -778,29 +830,86 @@ def copy_mol_props(
 
 def clear_mol_props(
     mol: Mol,
+    property_keys: Optional[Union[List[str], str]] = None,
     copy: bool = True,
     include_private: bool = False,
     include_computed: bool = False,
-):
-    """Clear all properties from a molecule.
+) -> Mol:
+    """Clear properties from a molecule.
 
     Args:
         mol: A molecule.
+        property_keys: If set, only the set properties will be cleared.
+            It not set, all the properties are cleared.
         copy: Whether to copy the molecule.
+        include_private: Whether to also clean the private properties.
+        include_computed: Whether to also clean the computed properties.
     """
 
     if copy:
         mol = copy_mol(mol)
 
-    props = mol.GetPropsAsDict(includePrivate=include_private, includeComputed=include_computed)
+    if property_keys is not None and isinstance(property_keys, str):
+        property_keys = [property_keys]
 
-    for key in props.keys():
+    if property_keys is None:
+        props = mol.GetPropsAsDict(includePrivate=include_private, includeComputed=include_computed)
+        property_keys = list(props.keys())
+
+    for key in property_keys:
         mol.ClearProp(key)
 
     return mol
 
 
-def atom_indices_to_mol(mol: Mol, copy: bool = False):
+def clear_atom_props(
+    mol: Mol,
+    property_keys: Optional[Union[List[str], str]] = None,
+    copy: bool = True,
+    include_private: bool = False,
+    include_computed: bool = False,
+) -> Mol:
+    """Clear atom properties from a molecule.
+
+    Args:
+        mol: A molecule.
+        property_keys: If set, only the set properties will be cleared.
+            It not set, all the properties are cleared.
+        copy: Whether to copy the molecule.
+        include_private: Whether to also clean the private properties.
+        include_computed: Whether to also clean the computed properties.
+    """
+
+    if copy:
+        mol = copy_mol(mol)
+
+    if property_keys is not None and isinstance(property_keys, str):
+        property_keys = [property_keys]
+
+    for atom in mol.GetAtoms():
+        if property_keys is None:
+            props = atom.GetPropsAsDict(
+                includePrivate=include_private, includeComputed=include_computed
+            )
+            property_keys = list(props.keys())
+
+        for key in property_keys:
+            atom.ClearProp(key)
+
+    return mol
+
+
+def clear_atom_map_number(mol: Mol, copy: bool = True):
+    """Clear the `molAtomMapNumber` property of the atom's molecule.
+
+    Args:
+        mol: A molecule.
+        copy: Whether to copy the molecule.
+    """
+    return clear_atom_props(mol, copy=copy, property_keys="molAtomMapNumber")
+
+
+def atom_indices_to_mol(mol: Mol, copy: bool = True):
     """Add the `molAtomMapNumber` property to each atoms.
 
     Args:
@@ -831,7 +940,7 @@ def atom_list_to_bond(
 
     # Build an atom map
     atom_map = {}
-    submol = PathToSubmol(mol, atom_indices, useQuery=True, atomMap=atom_map)
+    submol = rdmolops.PathToSubmol(mol, atom_indices, useQuery=True, atomMap=atom_map)
     atom_map_reversed = {v: k for k, v in atom_map.items()}
 
     bonds = []
@@ -885,7 +994,6 @@ def substructure_matching_bonds(mol: Mol, query: Mol, **kwargs: Any) -> Tuple[li
     bond_matches = []
 
     for match in atom_matches:
-
         # Map the atom of the query to the atom of the mol matching the query
         atom_map = dict(zip(query_atom_indices, match))
 
@@ -958,7 +1066,7 @@ def add_hs(
         add_residue_info: whether to add residue information to the hydrogens.
             Useful for PDB files.
     """
-    mol = AddHs(
+    mol = rdmolops.AddHs(
         mol,
         explicitOnly=explicit_only,
         addCoords=add_coords,
@@ -983,7 +1091,7 @@ def remove_hs(
         update_explicit_count: whether to update the explicit hydrogen count.
         sanitize: whether to sanitize the molecule after the hydrogens are removed.
     """
-    mol = RemoveHs(
+    mol = rdmolops.RemoveHs(
         mol,
         implicitOnly=implicit_only,
         updateExplicitCount=update_explicit_count,
@@ -1006,7 +1114,6 @@ def strip_mol_to_core(mol: Mol, bond_cutter: Mol = None):
         bond_cutter = from_smarts("[R;!$(*=,#[!#6])]!@!=!#[*;$([A;!R][A;!R])]")
 
     with without_rdkit_log():
-
         scaffold = MurckoScaffold.GetScaffoldForMol(mol)
         out = mol.GetSubstructMatches(bond_cutter)
         bond_inds = [mol.GetBondBetweenAtoms(i, j).GetIdx() for i, j in out]
@@ -1093,3 +1200,179 @@ def compute_ring_system(mol: Mol, include_spiro: bool = True) -> List[Set[int]]:
         nSystems.append(ringAts)
         systems = nSystems
     return systems
+
+
+def set_atom_positions(
+    mol: Mol,
+    positions: npt.ArrayLike,
+    conf_id: int = 0,
+    copy: bool = True,
+    use_atom_map_numbers: bool = True,
+    remove_previous_conformers: bool = True,
+) -> Mol:
+    """Add a conformer to a molecule given the atom's positions.
+
+    The conformer 3D flag is automatically set if all the z coordinates are 0.
+
+    **Example:**
+
+    The below example is common when you want to reconstruct a molecule object
+    from its SMILES and its held out atomic positions. The position array
+    is ordered according to the atom number seen in the SMILES. This is a common
+    data structure when working with Quantum Mechanics dataset.
+
+    ```python
+
+    import datamol as dm
+    import numpy as np
+
+    # We start with a SMILES where every atoms is mapped to a specific number
+    smiles = "[H:14][c:5]1[c:3]([c:7]([c:4]([c:6]([c:8]1[N:10]([H:18])[C:2](=[N+:11]([H:19])[H:20])[N:9]([H:16])[H:17])[H:15])[H:13])[F:1])[H:12]"
+
+    # Every atom position below is mapped to the atom number in the SMILES above
+    positions = [
+        [1.7, -6.67, 3.15],
+        [0.2, 4.72, 0.78],
+        [3.54, -2.64, 2.88],
+        [0.43, -3.87, -0.09],
+        [3.44, -0.2, 1.8],
+        [0.02, -1.5, -1.0],
+        [2.12, -4.54, 1.9],
+        [1.5, 0.48, 0.02],
+        [0.53, 7.24, 0.25],
+        [1.17, 2.91, -0.85],
+        [-1.22, 4.15, 2.71],
+        [4.64, -3.24, 4.55],
+        [-0.89, -5.43, -0.78],
+        [4.52, 1.43, 2.45],
+        [-1.45, -1.02, -2.48],
+        [-0.15, 8.68, 1.38],
+        [1.65, 7.88, -1.21],
+        [2.24, 3.64, -2.15],
+        [-1.96, 2.4, 3.0],
+        [-2.02, 5.59, 3.71],
+    ]
+
+    # We build the mol object by setting `remove_hs` to `False`.
+    # This is important so the hydrogens and their atom number are preserved.
+    mol = dm.to_mol(smiles, remove_hs=False)
+
+    # If you plot the molecule with `dm.to_image(mol)`, you'll notice
+    # the atom numbers are added to the drawing.
+
+    # Now we set the atom positions to the newly constructed `mol` object.
+    # Here it's important to set `use_atom_map_numbers` to `True`, so the atom numbers
+    # from the SMILES are used to match the input positions array.
+    # Under the hood, RDKit has set the `molAtomMapNumber` property to all the atoms in the
+    # molecule.
+    new_mol = dm.set_atom_positions(
+        mol=mol,
+        positions=positions,
+        conf_id=0,
+        use_atom_map_numbers=True,
+    )
+
+    # The newly set conformer now had the correct 3D positions.
+    # You can visualize the molecule with `dm.to_image(new_mol)` in 2D
+    # or `dm.viz.conformers(new_mol)`.
+    ```
+
+    Args:
+        mol: A molecule.
+        positions: An array or a list of atomic positions. Shape of `[n_atoms, 3]`.
+        conf_id: The conformer ID to set the conformer to.
+        copy: Whether to copy the molecule.
+        use_atom_map_numbers: Whether to input positions are ordered given the atom mapped
+            numbers set in the `molAtomMapNumber` atom property keys. If set to False,
+            the default atom indices order is assumed.
+        remove_previous_conformers: Whether to remove the previous conformers if any in the input molecule.
+    """
+
+    if copy:
+        mol = copy_mol(mol)
+
+    # Convert to an array
+    positions = np.array(positions)
+
+    # Perform a few sanity checks
+
+    if positions.ndim != 2:
+        raise ValueError(f"The positions array is not of dimension 2. Found: {positions.ndim}.")
+
+    excpected_shape = (mol.GetNumAtoms(), 3)
+    if positions.shape != excpected_shape:
+        raise ValueError(
+            f"The shape of `positions` is {positions.shape} instead of {excpected_shape}."
+        )
+
+    if use_atom_map_numbers and not all(
+        ["molAtomMapNumber" in a.GetPropsAsDict() for a in mol.GetAtoms()]
+    ):
+        raise ValueError(
+            f"The atoms of the input molecule does not contain the molAtomMapNumber property."
+            "Set it before calling this function or set `from_atom_map_numbers` to `False`."
+        )
+
+    # Remove previous conformers
+    if remove_previous_conformers:
+        mol.RemoveAllConformers()
+
+    # Remap the rows order in `positions` so it matches
+    # with the atom map numbers.
+    if use_atom_map_numbers:
+        mapped_indices = np.array([a.GetAtomMapNum() for a in mol.GetAtoms()]) - 1
+        positions = positions[mapped_indices, :]
+
+    # Check if it's 3D or 2D coords
+    is_3d = not np.all(positions[:, 2] == 0)
+
+    # Create the conformer object
+    conf = rdchem.Conformer()
+    conf.Set3D(is_3d)
+    conf.SetId(conf_id)
+
+    # Set the positions
+    for i, xyz in enumerate(positions):
+        conf.SetAtomPosition(i, rdGeometry.Point3D(*xyz.tolist()))
+
+    # Add the conformer to the molecule
+    mol.AddConformer(conf)
+
+    return mol
+
+
+def get_atom_positions(
+    mol: Mol,
+    conf_id: int = -1,
+    reorder_to_atom_map_number: bool = False,
+) -> np.ndarray:
+    """Return the atom positions of a given molecule.
+
+    Args:
+        mol: A molecule.
+        conf_id: The conformer ID to set the conformer to.
+        reorder_to_atom_map_number: Whether to reorder the positions to map the
+            atom map numbers given by the `molAtomMapNumber` atom property.
+    """
+
+    if mol.GetNumConformers() == 0:
+        raise ValueError("This molecule does not have conformers.")
+
+    if reorder_to_atom_map_number and not all(
+        ["molAtomMapNumber" in a.GetPropsAsDict() for a in mol.GetAtoms()]
+    ):
+        raise ValueError(
+            f"The atoms of the input molecule does not contain the molAtomMapNumber property."
+            "Set it before calling this function or set `from_atom_map_numbers` to `False`."
+        )
+
+    conformer = mol.GetConformer(id=conf_id)
+    positions = conformer.GetPositions()
+
+    if reorder_to_atom_map_number:
+        # Remap the rows order in `positions` so it matches
+        # with the atom map numbers.
+        mapped_indices = np.array([a.GetAtomMapNum() for a in mol.GetAtoms()]) - 1
+        positions = positions[mapped_indices, :]
+
+    return positions
