@@ -21,19 +21,20 @@ def prepare_mol_for_drawing(mol: Optional[dm.Mol], kekulize: bool = True) -> Opt
 
     try:
         with dm.without_rdkit_log():
-
             # Check for implicit and explicit valence
             if mol.NeedsUpdatePropertyCache():  # type: ignore
                 mol.UpdatePropertyCache(False)  # type: ignore
 
             # Check for aromaticity
-            _kekulize = Draw._okToKekulizeMol(mol, kekulize)
+            if dm.is_lower_than_current_rdkit_version("2022.09"):
+                _kekulize = Draw._okToKekulizeMol(mol, kekulize)
+            else:
+                _kekulize = Draw.shouldKekulize(mol, kekulize)
 
             # Run the rdkit preparation procedure
             _mol = Draw.rdMolDraw2D.PrepareMolForDrawing(mol, kekulize=_kekulize)
 
     except ValueError:  # <- can happen on a kekulization failure
-
         # Run the rdkit preparation procedure with kekulize set to `False`
         _mol = Draw.rdMolDraw2D.PrepareMolForDrawing(mol, kekulize=False)
 
